@@ -4,6 +4,22 @@ import click
 
 
 def test_other_command_invoke(runner):
+    @click.command()
+    @click.pass_context
+    def cli(ctx):
+        return ctx.invoke(other_cmd, arg=42)
+
+    @click.command()
+    @click.argument('arg', type=click.INT)
+    def other_cmd(arg):
+        click.echo(arg)
+
+    result = runner.invoke(cli, [])
+    assert not result.exception
+    assert result.output == '42\n'
+
+
+def test_other_command_invoke_invalid_custom_error(runner):
     @click.command()
     @click.pass_context
     def cli(ctx):
@@ -15,8 +31,9 @@ def test_other_command_invoke(runner):
         click.echo(arg)
 
     result = runner.invoke(cli, [])
-    assert not result.exception
-    assert result.output == '42\n'
+    assert isinstance(result.exception, RuntimeError)
+    assert 'upgrading-to-3.2' in str(result.exception)
+    assert click.__version__ < '5.0'
 
 
 def test_other_command_forward(runner):
@@ -194,3 +211,62 @@ def test_object_propagation(runner):
         result = runner.invoke(cli, ['sync'])
         assert result.exception is None
         assert result.output == 'Debug is off\n'
+
+
+def test_other_command_invoke_with_defaults(runner):
+    @click.command()
+    @click.pass_context
+    def cli(ctx):
+        return ctx.invoke(other_cmd)
+
+    @click.command()
+    @click.option('--foo', type=click.INT, default=42)
+    @click.pass_context
+    def other_cmd(ctx, foo):
+        assert ctx.info_name == 'other_cmd'
+        click.echo(foo)
+
+    result = runner.invoke(cli, [])
+    assert not result.exception
+    assert result.output == '42\n'
+
+
+def test_invoked_subcommand(runner):
+    @click.group(invoke_without_command=True)
+    @click.pass_context
+    def cli(ctx):
+        if ctx.invoked_subcommand is None:
+            click.echo('no subcommand, use default')
+            ctx.invoke(sync)
+        else:
+            click.echo('invoke subcommand')
+
+    @cli.command()
+    def sync():
+        click.echo('in subcommand')
+
+    result = runner.invoke(cli, ['sync'])
+    assert not result.exception
+    assert result.output == 'invoke subcommand\nin subcommand\n'
+
+    result = runner.invoke(cli)
+    assert not result.exception
+    assert result.output == 'no subcommand, use default\nin subcommand\n'
+
+
+def test_unprocessed_options(runner):
+    @click.command(context_settings=dict(
+        ignore_unknown_options=True
+    ))
+    @click.argument('args', nargs=-1, type=click.UNPROCESSED)
+    @click.option('--verbose', '-v', count=True)
+    def cli(verbose, args):
+        click.echo('Verbosity: %s' % verbose)
+        click.echo('Args: %s' % '|'.join(args))
+
+    result = runner.invoke(cli, ['-foo', '-vvvvx', '--muhaha', 'x', 'y', '-x'])
+    assert not result.exception
+    assert result.output.splitlines() == [
+        'Verbosity: 4',
+        'Args: -foo|-x|--muhaha|x|y|-x',
+    ]
